/* Copyright (C) 1997 Aladdin Enterprises.  All rights reserved.
  
  This file is part of Aladdin Ghostscript.
  
  Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
  or distributor accepts any responsibility for the consequences of using it,
  or for whether it serves any particular purpose or works at all, unless he
  or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
  License (the "License") for full details.
  
  Every copy of Aladdin Ghostscript must include a copy of the License,
  normally in a plain ASCII text file named PUBLIC.  The License grants you
  the right to copy, modify and redistribute Aladdin Ghostscript, but only
  under certain conditions described in the License.  Among other things, the
  License requires that the copyright notice and this notice be preserved on
  all copies.
*/

/* gsfunc0.h */
/* Definitions for FunctionType 0 (Sampled) Functions */

#ifndef gsfunc0_INCLUDED
#  define gsfunc0_INCLUDED

#include "gsfunc.h"
#include "gsdsrc.h"

/* ---------------- Types and structures ---------------- */

/* Define the Function type. */
#define function_type_Sampled 0

/* Define Sampled functions. */
typedef struct gs_function_Sd_params_s {
  gs_function_params_common;
  int Order;			/* 1 or 3, optional */
  gs_data_source_t DataSource;
  int BitsPerSample;		/* 1, 2, 4, 8, 12, 16, 24, 32 */
  const float *Encode;		/* 2 x m, optional */
  const float *Decode;		/* 2 x n, optional */
  const int *Size;		/* m */
} gs_function_Sd_params_t;
#define private_st_function_Sd()	/* in gsfunc.c */\
  gs_private_st_suffix_add3(st_function_Sd, gs_function_Sd_t,\
    "gs_function_Sd_t", function_Sd_enum_ptrs, function_Sd_reloc_ptrs,\
    st_function, params.Encode, params.Decode, params.Size)

/* ---------------- Procedures ---------------- */

/* Allocate and initialize a Sampled function. */
int gs_function_Sd_init(P3(gs_function_t **ppfn,
			   const gs_function_Sd_params_t *params,
			   gs_memory_t *mem));

/* Free the parameters of a Sampled function. */
void gs_function_Sd_free_params(P2(gs_function_Sd_params_t *params,
				   gs_memory_t *mem));

#endif					/* gsfunc0_INCLUDED */
